==> pytchat-0.2.6-py3-none-any.whl: pytchat/parser/live.py <==
"""
pytchat.parser.live
~~~~~~~~~~~~~~~~~~~
Parser of live chat JSON.
"""

from .. import exceptions


class Parser:

    __slots__ = ['is_replay']

    def __init__(self, is_replay):
        self.is_replay = is_replay

    def get_contents(self, jsn):
        if jsn is None:
            raise exceptions.IllegalFunctionCall('Called with none JSON object.')
        if jsn['response']['responseContext'].get('errors'):
            raise exceptions.ResponseContextError(
                'The video_id would be wrong, or video is deleted or private.')
        contents = jsn['response'].get('continuationContents')
        return contents

    def parse(self, contents):
        """
        Parameter
        ----------
        + contents : dict
            + JSON of chat data from YouTube.

        Returns
        -------
        tuple:
        + metadata : dict
         + timeout
         + video_id
         + continuation
        + chatdata : List[dict]
        """

        if contents is None:
            '''Broadcasting end or cannot fetch chat stream'''
            raise exceptions.NoContents('Chat data stream is empty.')

        cont = contents['liveChatContinuation']['continuations'][0]
        if cont is None:
            raise exceptions.NoContinuation('No Continuation')
        metadata = (cont.get('invalidationContinuationData')
                    or cont.get('timedContinuationData')
                    or cont.get('reloadContinuationData')
                    or cont.get('liveChatReplayContinuationData')
                    )
        if metadata is None:
            if cont.get("playerSeekContinuationData"):
                raise exceptions.ChatDataFinished('Finished chat data')
            unknown = list(cont.keys())[0]
            if unknown:
                raise exceptions.ReceivedUnknownContinuation(
                    f"Received unknown continuation type:{unknown}")
            else:
                raise exceptions.FailedExtractContinuation('Cannot extract continuation data')
        return self._create_data(metadata, contents)

    def reload_continuation(self, contents):
        """
        When `seektime == 0` or seektime is abbreviated ,
        check if fetched chat json has no chat data.
        If so, try to fetch playerSeekContinuationData.
        This function must be run only first fetching.
        """
        if contents is None:
            '''Broadcasting end or cannot fetch chat stream'''
            raise exceptions.NoContents('Chat data stream is empty.')
        cont = contents['liveChatContinuation']['continuations'][0]
        if cont.get("liveChatReplayContinuationData"):
            # chat data exist.
            return None
        # chat data do not exist, get playerSeekContinuationData.
        init_cont = cont.get("playerSeekContinuationData")
        if init_cont:
            return init_cont.get("continuation")
        raise exceptions.ChatDataFinished('Finished chat data')

    def _create_data(self, metadata, contents):
        actions = contents['liveChatContinuation'].get('actions')
        if self.is_replay:
            interval = self._get_interval(actions)
            metadata.setdefault("timeoutMs", interval)
            """Archived chat has different structures than live chat,
            so make it the same format."""
            chatdata = [action["replayChatItemAction"]["actions"][0]
                        for action in actions]
        else:
            metadata.setdefault('timeoutMs', 10000)
            chatdata = actions
        return metadata, chatdata

    def _get_interval(self, actions: list):
        if actions is None:
            return 0
        start = int(actions[0]["replayChatItemAction"]["videoOffsetTimeMsec"])
        last = int(actions[-1]["replayChatItemAction"]["videoOffsetTimeMsec"])
        return (last - start)

==> pytchat-0.4.0-py3-none-any.whl: pytchat/parser/live.py <==
"""
pytchat.parser.live
~~~~~~~~~~~~~~~~~~~
Parser of live chat JSON.
"""

from .. import exceptions


class Parser:
    '''
    Parser of chat json.
    
    Parameter
    ----------
    is_replay : bool

    exception_holder : Object [default:Npne]
        The object holding exceptions.
        This is passed from the parent livechat object.
    '''
    __slots__ = ['is_replay', 'exception_holder']

    def __init__(self, is_replay, exception_holder=None):
        self.is_replay = is_replay
        self.exception_holder = exception_holder

    def get_contents(self, jsn):
        if jsn is None:
            self.raise_exception(exceptions.IllegalFunctionCall('Called with none JSON object.'))
        if jsn['response']['responseContext'].get('errors'):
            raise exceptions.ResponseContextError(
                'The video_id would be wrong, or video is deleted or private.')
        contents = jsn['response'].get('continuationContents')
        return contents

    def parse(self, contents):
        """
        Parameter
        ----------
        + contents : dict
            + JSON of chat data from YouTube.

        Returns
        -------
        tuple:
        + metadata : dict
         + timeout
         + video_id
         + continuation
        + chatdata : List[dict]
        """

        if contents is None:
            '''Broadcasting end or cannot fetch chat stream'''
            self.raise_exception(exceptions.NoContents('Chat data stream is empty.'))

        cont = contents['liveChatContinuation']['continuations'][0]
        if cont is None:
            self.raise_exception(exceptions.NoContinuation('No Continuation'))
        metadata = (cont.get('invalidationContinuationData')
                    or cont.get('timedContinuationData')
                    or cont.get('reloadContinuationData')
                    or cont.get('liveChatReplayContinuationData')
                    )
        if metadata is None:
            if cont.get("playerSeekContinuationData"):
                self.raise_exception(exceptions.ChatDataFinished('Finished chat data'))
            unknown = list(cont.keys())[0]
            if unknown:
                self.raise_exception(exceptions.ReceivedUnknownContinuation(
                    f"Received unknown continuation type:{unknown}"))
            else:
                self.raise_exception(exceptions.FailedExtractContinuation('Cannot extract continuation data'))
        return self._create_data(metadata, contents)

    def reload_continuation(self, contents):
        """
        When `seektime == 0` or seektime is abbreviated ,
        check if fetched chat json has no chat data.
        If so, try to fetch playerSeekContinuationData.
        This function must be run only first fetching.
        """
        if contents is None:
            '''Broadcasting end or cannot fetch chat stream'''
            self.raise_exception(exceptions.NoContents('Chat data stream is empty.'))
        cont = contents['liveChatContinuation']['continuations'][0]
        if cont.get("liveChatReplayContinuationData"):
            # chat data exist.
            return None
        # chat data do not exist, get playerSeekContinuationData.
        init_cont = cont.get("playerSeekContinuationData")
        if init_cont:
            return init_cont.get("continuation")
        self.raise_exception(exceptions.ChatDataFinished('Finished chat data'))

    def _create_data(self, metadata, contents):
        actions = contents['liveChatContinuation'].get('actions')
        if self.is_replay:
            interval = self._get_interval(actions)
            metadata.setdefault("timeoutMs", interval)
            """Archived chat has different structures than live chat,
            so make it the same format."""
            chatdata = [action["replayChatItemAction"]["actions"][0]
                        for action in actions]
        else:
            metadata.setdefault('timeoutMs', 10000)
            chatdata = actions
        return metadata, chatdata

    def _get_interval(self, actions: list):
        if actions is None:
            return 0
        start = int(actions[0]["replayChatItemAction"]["videoOffsetTimeMsec"])
        last = int(actions[-1]["replayChatItemAction"]["videoOffsetTimeMsec"])
        return (last - start)

    def raise_exception(self, exception):
        if self.exception_holder is None:
            raise exception
        self.exception_holder = exception
